@@ -19,7 +19,7 @@ import zlib
 from wpull.actor import Event
 from wpull.errors import (SSLVerficationError, ConnectionRefused, NetworkError,
     ProtocolError)
-from wpull.extended import SSLIOStream, IOStream
+from wpull.iostream import SSLIOStream, IOStream, BufferFullError
 from wpull.http.request import Response
 from wpull.network import Resolver
 
@@ -73,7 +73,7 @@ class Connection(object):
             self.request_data.clear()
             self.response_data.clear()
 
-    DEFAULT_BUFFER_SIZE = 10485760
+    DEFAULT_BUFFER_SIZE = 1048576
     '''Default buffer size in bytes.'''
     DEFAULT_NO_CONTENT_CODES = frozenset(itertools.chain(
         range(100, 200),
@@ -120,16 +120,15 @@ class Connection(object):
             self._io_stream = SSLIOStream(
                 self._socket,
                 max_buffer_size=self._buffer_size,
-                connect_timeout=self._connect_timeout,
-                read_timeout=self._read_timeout,
+                rw_timeout=self._read_timeout,
                 ssl_options=self._ssl_options,
+                server_hostname=self._host,
             )
         else:
             self._io_stream = IOStream(
                 self._socket,
+                rw_timeout=self._read_timeout,
                 max_buffer_size=self._buffer_size,
-                connect_timeout=self._connect_timeout,
-                read_timeout=self._read_timeout,
             )
 
         self._io_stream.set_close_callback(self._stream_closed_callback)
@@ -146,7 +145,9 @@ class Connection(object):
 
         _logger.debug('Connecting to {0}.'.format(self._address))
         try:
-            yield self._io_stream.connect(self._address, self._host)
+            yield self._io_stream.connect(
+                self._address, timeout=self._connect_timeout
+            )
         except (ssl.SSLError, tornado.netutil.SSLCertificateError,
         SSLVerficationError) as error:
             raise SSLVerficationError('SSLError: {error}'.format(
@@ -160,7 +161,6 @@ class Connection(object):
                     error=error)) from error
         else:
             _logger.debug('Connected.')
-            self._connected = True
 
     @tornado.gen.coroutine
     def fetch(self, request, recorder=None, response_factory=Response):
@@ -205,8 +205,8 @@ class Connection(object):
             self._events.clear()
             self._active = False
 
-        if not self._keep_alive:
-            _logger.debug('Closing connection.')
+        if not self._keep_alive and self.connected:
+            _logger.debug('Not keep-alive. Closing connection.')
             self.close()
 
         _logger.debug('Fetching done.')
@@ -226,9 +226,9 @@ class Connection(object):
         self._events.pre_request(request)
 
         if sys.version_info < (3, 3):
-            error_class = (socket.error, StreamClosedError)
+            error_class = (socket.error, StreamClosedError, ssl.SSLError)
         else:
-            error_class = (ConnectionError, StreamClosedError)
+            error_class = (ConnectionError, StreamClosedError, ssl.SSLError)
 
         try:
             yield self._send_request_header(request)
@@ -241,11 +241,29 @@ class Connection(object):
             yield self._read_response_body(request, response)
         except error_class as error:
             raise NetworkError('Network error: {0}'.format(error)) from error
+        except BufferFullError as error:
+            raise ProtocolError(*error.args) from error
 
         self._events.response.fire(response)
 
+        if self.should_close(
+        request.version, response.fields.get('Connection')):
+            _logger.debug('HTTP connection close.')
+            self.close()
+        else:
+            self._io_stream.monitor_for_close()
+
         raise tornado.gen.Return(response)
 
+    @classmethod
+    def should_close(cls, http_version, connection_field):
+        connection_field = (connection_field or '').lower()
+
+        if http_version == 'HTTP/1.0':
+            return connection_field.replace('-', '') != 'keepalive'
+        else:
+            return connection_field == 'close'
+
     @tornado.gen.coroutine
     def _send_request_header(self, request):
         '''Send the request's HTTP status line and header fields.'''
@@ -266,8 +284,10 @@ class Connection(object):
     def _read_response_header(self, response_factory):
         '''Read the response's HTTP status line and header fields.'''
         _logger.debug('Reading header.')
+
         response_header_data = yield self._io_stream.read_until_regex(
-            br'\r?\n\r?\n')
+            br'\r?\n\r?\n'
+        )
 
         self._events.response_data.fire(response_header_data)
 
@@ -355,17 +375,14 @@ class Connection(object):
             yield self._read_response_until_close(response)
             return
 
-        data_queue = self._io_stream.read_bytes_queue(body_size)
-
-        while True:
-            data = yield data_queue.get()
-
-            if data is None:
-                break
-
+        def callback(data):
             self._events.response_data.fire(data)
             response.body.content_file.write(self._decompress_data(data))
 
+        yield self._io_stream.read_bytes(
+            body_size, streaming_callback=callback,
+        )
+
         response.body.content_file.write(self._flush_decompressor())
 
     @tornado.gen.coroutine
@@ -394,17 +411,12 @@ class Connection(object):
         '''Read the response until the connection closes.'''
         _logger.debug('Reading body until close.')
 
-        data_queue = self._io_stream.read_until_close_queue()
-
-        while True:
-            data = yield data_queue.get()
-
-            if data is None:
-                break
-
+        def callback(data):
             self._events.response_data.fire(data)
             response.body.content_file.write(self._decompress_data(data))
 
+        yield self._io_stream.read_until_close(streaming_callback=callback)
+
         response.body.content_file.write(self._flush_decompressor())
 
     @property
@@ -424,30 +436,14 @@ class Connection(object):
             self._io_stream.close()
 
     def _stream_closed_callback(self):
-        _logger.debug('Stream closed. '
-            'active={0} connected={1} ' \
-            'closed={2} reading={3} writing={3}'.format(
+        _logger.debug(
+            'Stream closed. active={0} connected={1} closed={2}'.format(
                 self._active,
                 self.connected,
                 self._io_stream.closed(),
-                self._io_stream.reading(),
-                self._io_stream.writing())
+            )
         )
 
-        if not self._active:
-            # We are likely in a context that's already dead
-            _logger.debug('Ignoring stream closed error={0}.'\
-                .format(self._io_stream.error))
-            return
-
-        if self._io_stream.error:
-            _logger.debug('Throwing error {0}.'.format(self._io_stream.error))
-            raise self._io_stream.error
-
-        if self._io_stream.buffer_full:
-            _logger.debug('Buffer full.')
-            raise ProtocolError('Buffer full.')
-
 
 class ChunkedTransferStreamReader(object):
     '''Read chunked transfer encoded stream.
@@ -488,17 +484,14 @@ class ChunkedTransferStreamReader(object):
         if not chunk_size:
             raise tornado.gen.Return(chunk_size)
 
-        data_queue = self._io_stream.read_bytes_queue(chunk_size)
-
-        while True:
-            data = yield data_queue.get()
-
-            if data is None:
-                break
-
+        def callback(data):
             self.data_event.fire(data)
             self.content_event.fire(data)
 
+        yield self._io_stream.read_bytes(
+            chunk_size, streaming_callback=callback
+        )
+
         newline_data = yield self._io_stream.read_until(b'\n')
 
         self.data_event.fire(newline_data)
@@ -591,7 +584,7 @@ class HostConnectionPool(collections.Set):
             yield self._process_request()
             self._max_count_semaphore.release()
             _logger.debug('Host pool semaphore released.')
-        except:
+        except Exception:
             _logger.exception('Fatal error processing request.')
             sys.exit('Fatal error.')
 
@@ -682,11 +675,13 @@ class ConnectionPool(collections.Mapping):
 
         if request.address:
             address = request.address
+            host, port = address
         else:
             host = request.url_info.hostname
             port = request.url_info.port
             address = (host, port)
-            ssl = (request.url_info.scheme == 'https')
+
+        ssl = (request.url_info.scheme == 'https')
 
         if address not in self._pools:
             _logger.debug('New host pool.')
